%run me

fl = 24;
QF = 17.9;              %24mm 17.9 28mm 20.1 35mm 20.1 50mm 20.4 70mm 20.4
d = [1 2 3 5 7 10 15 20 30 50];
plotFlag = true;

N = 2.^(2:0.5/3:4.5);
ns = 20;                                %distance samples in decade
u2 = (10.^(1/ns)).^(-30:61);            %m
th = 50;                                %lp/mm, MTF50 'acceptable' sharpness

ln = length(N);
ld = length(d);
mMax = zeros(ld,ln);
mMaxu = zeros(ld,ln);
un = zeros(ld,ln);
uf = zeros(ld,ln);
m1k = zeros(ld,ln);
un1k = zeros(ld,ln);
S = zeros(ld,6);
D = zeros(ld,6);
C = zeros(ld,6);
mtf50s = zeros(ld,ln,length(u2));

for j = 1:ld
    for i = 1:ln
        fprintf('.')
        [u2,mtf50] = DOFMTF50(fl,N(i),d(j),false,u2,QF);
        mtf50 = mtf50*1000;                 %lp/mm
        
        [mMax(j,i),mMaxu(j,i),un(j,i),uf(j,i),m1k(j,i),un1k(j,i)] =...
            dofStats(u2,mtf50,th);
        
        mtf50s(j,i,:) = mtf50;
    end
    fprintf(' Focus Distance %d m:\r',d(j))
    
    [M,I] = max(mMax(j,:));                 %Focus at Subject
    S(j,1) = N(I);
    S(j,2) = mMax(j,I);
    S(j,3) = m1k(j,I);
    S(j,4) = uf(j,I);
    S(j,5) = un(j,I);
    S(j,6) = un1k(j,I);
    fprintf(['Best In-focus   f/%3.1f, Peak %3.1f lp/mm, Vanish %3.1f, Far %.3fm'...
        ', Near %.3fm, Near V %.3fm\n'],N(I),mMax(j,I),m1k(j,I),uf(j,I),un(j,I),un1k(j,I));
    
    [~,I] = max(m1k(j,:));                  %Focus at a Distance
    D(j,1) = N(I);
    D(j,2) = mMax(j,I);
    D(j,3) = m1k(j,I);
    D(j,4) = uf(j,I);
    D(j,5) = un(j,I);
    D(j,6) = un1k(j,I);
    fprintf(['Best Vanish     f/%3.1f, Peak %3.1f lp/mm, Vanish %3.1f, Far %.3fm'...
        ', Near %.3fm, Near V %.3fm\n'],N(I),mMax(j,I),m1k(j,I),uf(j,I),un(j,I),un1k(j,I));
    
    [~,I] = min(un(j,:));                   %Nearest Limit
    C(j,1) = N(I);
    C(j,2) = mMax(j,I);
    C(j,3) = m1k(j,I);
    C(j,4) = uf(j,I);
    C(j,5) = un(j,I);
    C(j,6) = un1k(j,I);
    fprintf(['Best Near Limit f/%3.1f, Peak %3.1f lp/mm, Vanish %3.1f, Far %.3fm'...
        ', Near %.3fm, Near V %.3fm\n'],N(I),mMax(j,I),m1k(j,I),uf(j,I),un(j,I),un1k(j,I));
    
    if plotFlag
        figure; set(gcf,'WindowStyle','docked')
        plot(N,uf(j,:),'.-k',N,un(j,:),'k',N,un1k(j,:),'--k')
        set(gca,'xscale','log'); grid
        xlabel('f-number'), ylabel('Distance to Threshold (m)')
        xticks(N); xticklabels(split(num2str(N,'%5.1f')))
        yticks([0.1:0.1:0.5 0.7 1:5 7 10 15 20 30 40 60 100 150 200 300 400 600 1000])
        set(gca,'yscale','log'); ylim([1 1000])
        
        yyaxis right
        plot(N,mMax(j,:),'-','color',[0 0.4470 0.7410]); hold on
        plot(N,m1k(j,:),'--','color',[0 0.4470 0.7410])
        plot(N,th*ones(1,ln),'-','color',[0 0.4470 0.7410],'LineWidth',.2)
        ylim([0 100]); ylabel('MTF50 (lp/mm)')
        legend(['Far   ' num2str(th) 'lp/mm Limit'],['Near ' num2str(th) 'lp/mm Limit'],...
            'Near same as 1k','In-focus MTF50','1Km MTF50')
        title(['Z7+24-70mm/4, fl ' num2str(fl) 'mm focused at ' num2str(d(j)) 'm, '...
            'MTF50 Thresh = ' num2str(th) ' lp/mm'])
        yyaxis left; hold off
    end
end

figure
plot(d,S(:,1),'color',[0 0.4470 0.7410]);hold on
plot(d,S(:,1)*2^(1/6),'color',[0 0.4470 0.7410])
plot(d,D(:,1),'color',[0.8500 0.3250 0.0980])
plot(d,D(:,1)*2^(-1/6),'color',[0.8500 0.3250 0.0980])

set(gca,'xscale','log');
xticks(d); xlim([1 50])
xlabel('Focus Distance (m)')
xticks([1:10 12 15 17 20:5:40 50])
xticklabels([1:10 12 15 17 20:5:40 50])
set(gca,'yscale','log');
yticks(N); yticklabels(split(num2str(N,'%5.1f')))
ylim([4 22]); ylabel('f/stop')

yyaxis right
plot(d,C(:,5),'--');                                %f/10 Near Limit
plot(d,S(:,5),'--','color',[0 0.4470 0.7410])       %Subject Near Limit
plot(d,D(:,5),'--','color',[0.8500 0.3250 0.0980])  %Distance Near Limit
title(['Aperture-Distance Sharpness Chart, Z7+24-70mm/4 S at ' num2str(fl)...
    'mm'])
ylabel('Distance from Lens to Threshold (m)'); grid on

figure
plot(d,un(:,4)','k'); hold on
plot(d,un(:,7)','--k')
plot(d,un(:,10)',':k')
ylim([0 18])
ylabel('Near Limit (m)'); xlabel('Focus Distance (m)')
yyaxis right; plot(d,m1k(:,4:3:10))
ylim([-20 70])
ylabel('Resolution at 1 Km (lp/mm)')
grid on; grid minor
legend('f/5.6','f/8','f/11','Location','NW')
xlim([0 25])
title(['Near-Limit/Far-Sharpness Trade-offs, Z7+kit at ' num2str(fl) 'mm'])

